/*:
-------------------------------------------------------------------------------
@title Hime Inventory System
@author Hime --> HimeWorks (http://himeworks.com)
@version 1.0
@date Sep 2, 2020
@filename HIME_InventorySystem.js
@url 

If you enjoy my work, consider supporting me on Patreon!

* https://www.patreon.com/himeworks

If you have any questions or concerns, you can contact me at any of
the following sites:

* Main Website: http://himeworks.com
* Facebook: https://www.facebook.com/himeworkscom/
* Twitter: https://twitter.com/HimeWorks
* Youtube: https://www.youtube.com/c/HimeWorks
* Tumblr: http://himeworks.tumblr.com/

-------------------------------------------------------------------------------
@plugindesc v1.0 - Changes the way inventory management is handled and allows
you to switch between inventories quickly
@help 
-------------------------------------------------------------------------------
== Description ==


== Terms of Use ==

- Free for use in non-commercial projects with credits
- Free for use in commercial projects, but it would be nice to let me know
- Please provide credits to HimeWorks

== Change Log ==

== Usage ==

To switch party inventories, use the script call

  $gameParty.setInventory( ID )

Where the ID is the ID of the inventory you want to switch to.
If the ID doesn't exist, a new inventory will be created with that ID.

By default the first inventory is called "party1", so for example let's say
you wanted to create a second party inventory called "party2", you would say

  $gameParty.setInventory("party2")
  
And then when you want to switch back to the first inventory, you would say

  $gameParty.setInventory("party1")

-------------------------------------------------------------------------------
*/ 
var Imported = Imported || {} ;
var TH = TH || {};
Imported.TH_InventorySystem = 1;
TH.InventorySystem = TH.InventorySystem || {}; 


function Game_Inventories() {
  this.initialize.apply(this, arguments);
}

// an inventory object
function Game_Inventory() {
  this.initialize.apply(this, arguments);
}

Game_Inventory.prototype.initialize = function(id) {
  this._id = id
  this._items = {}
  this._weapons = {}
  this._armors = {}
  this._gold = 0
};

(function ($) {  

  var TH_DataManager_createGameObjects = DataManager.createGameObjects
  DataManager.createGameObjects = function() {
    $gameInventories = new Game_Inventories();
    TH_DataManager_createGameObjects.call(this);    
  }
  
  var TH_DataManager_makeSaveContents = DataManager.makeSaveContents
  DataManager.makeSaveContents = function() {
    contents = TH_DataManager_makeSaveContents.call(this);
    contents.inventories       = $gameInventories;
    return contents;
  }
  
  var TH_DataManager_extractSaveContents = DataManager.extractSaveContents
  DataManager.extractSaveContents = function(contents) {
    TH_DataManager_extractSaveContents.call(this, contents);
    $gameInventories = contents.inventories;
  }

  Game_Inventories.prototype.initialize = function() {
    this._inventories = {}
  }

  Game_Inventories.prototype.create = function(id) {
    this._inventories[id] = new Game_Inventory(id);
  }

  Game_Inventories.prototype.get = function(id) {
    if (!this._inventories[id]) {
      this.create(id);
    }
    return this._inventories[id];
  }
  
  /*************************************************************/
  
  Game_Inventory.prototype.gold = function() {
    return this._gold;
  };

  Game_Inventory.prototype.gainGold = function(amount) {
      this._gold = (this._gold + amount).clamp(0, this.maxGold());
  };

  Game_Inventory.prototype.loseGold = function(amount) {
      this.gainGold(-amount);
  };

  Game_Inventory.prototype.maxGold = function() {
      return 99999999;
  };
  
  Game_Inventory.prototype.itemContainer = function(item) {
    if (!item) {
        return null;
    } else if (DataManager.isItem(item)) {
        return this._items;
    } else if (DataManager.isWeapon(item)) {
        return this._weapons;
    } else if (DataManager.isArmor(item)) {
        return this._armors;
    } else {
        return null;
    }
  };
  
  Game_Inventory.prototype.items = function() {
      var list = [];
      for (var id in this._items) {
          list.push($dataItems[id]);
      }
      return list;
  };

  Game_Inventory.prototype.weapons = function() {
      var list = [];
      for (var id in this._weapons) {
          list.push($dataWeapons[id]);
      }
      return list;
  };

  Game_Inventory.prototype.armors = function() {
      var list = [];
      for (var id in this._armors) {
          list.push($dataArmors[id]);
      }
      return list;
  };

  Game_Inventory.prototype.equipItems = function() {
      return this.weapons().concat(this.armors());
  };

  Game_Inventory.prototype.allItems = function() {
      return this.items().concat(this.equipItems());
  };
  
  Game_Inventory.prototype.numItems = function(item) {
    var container = this.itemContainer(item);
    return container ? container[item.id] || 0 : 0;
  }
  
  Game_Inventory.prototype.maxItems = function(item) {
    return 99;
  }
  
  Game_Inventory.prototype.hasMaxItems = function(item) {
      return this.numItems(item) >= this.maxItems(item);
  };

  Game_Inventory.prototype.hasItem = function(item, includeEquip) {
      if (includeEquip === undefined) {
          includeEquip = false;
      }
      if (this.numItems(item) > 0) {
          return true;
      } else if (includeEquip && this.isAnyMemberEquipped(item)) {
          return true;
      } else {
          return false;
      }
  };

  Game_Inventory.prototype.gainItem = function(item, amount, includeEquip) {
      var container = this.itemContainer(item);
      if (container) {
          var lastNumber = this.numItems(item);
          var newNumber = lastNumber + amount;
          container[item.id] = newNumber.clamp(0, this.maxItems(item));
          if (container[item.id] === 0) {
              delete container[item.id];
          }
          if (includeEquip && newNumber < 0) {
              this.discardMembersEquip(item, -newNumber);
          }
          $gameMap.requestRefresh();
      }
  };
  
  /*************************************************************/
  
  var TH_GameParty_initialize = Game_Party.prototype.initialize
  Game_Party.prototype.initialize = function() {   
    TH_GameParty_initialize.call(this);
    this.createInventory();
  }
  
  Game_Party.prototype.defaultInventoryId = function() {
    return "party1";
  }

  Game_Party.prototype.createInventory = function() {
    this._inventory = $gameInventories.get(this.defaultInventoryId());
  };
  
  Game_Party.prototype.setInventory = function(id) {
    this._inventory = $gameInventories.get(id);
  }
  
  Game_Party.prototype.getInventory = function() {
    return this._inventory;
  }
  
  Game_Party.prototype.gold = function() {
    return this.getInventory().gold();
  };

  Game_Party.prototype.gainGold = function(amount) {
    this.getInventory().gainGold(amount);
  };

  Game_Party.prototype.loseGold = function(amount) {
    this.getInventory().loseGold(amount);
  };

  Game_Party.prototype.maxGold = function() {
    return this.getInventory().maxGold();
  };
    
  Game_Party.prototype.items = function() {
    return this.getInventory().items();
  };

  Game_Party.prototype.weapons = function() {
    return this.getInventory().weapons();
  };

  Game_Party.prototype.armors = function() {
    return this.getInventory().armors();
  };

  Game_Party.prototype.equipItems = function() {
    return this.getInventory().equipItems()
  };

  Game_Party.prototype.allItems = function() {
    return this.getInventory().allItems();
  };

  /* Redirect part */
  Game_Party.prototype.numItems = function(item) {
    return this.getInventory().numItems(item);
  };

  Game_Party.prototype.maxItems = function(item) {
    return this.getInventory().maxItems(item);
  };

  Game_Party.prototype.hasMaxItems = function(item) {
    return this.getInventory().hasMaxItems(item);
  };

  // overwrite
  Game_Party.prototype.hasItem = function(item, includeEquip) {
      if (includeEquip === undefined) {
          includeEquip = false;
      }
      if (this.getInventory().numItems(item) > 0) {
          return true;
      } else if (includeEquip && this.isAnyMemberEquipped(item)) {
          return true;
      } else {
          return false;
      }
  };

  Game_Party.prototype.gainItem = function(item, amount, includeEquip) {
    this.getInventory().gainItem(item, amount, includeEquip)    
    if (includeEquip && newNumber < 0) {
      this.discardMembersEquip(item, -newNumber);
    }
  };

  Game_Party.prototype.loseItem = function(item, amount, includeEquip) {
      this.gainItem(item, -amount, includeEquip);
  };


})(TH.InventorySystem);
