// =================================================================== //
// ==========		Plugin Provided by ShadowDragon			========== //
// ==========     ShadowDragon Battle Messages Plugin  	    ========== //
// =================================================================== //
/** /*:
*
* @author ShadowDragon
* @plugindesc set random Battle Messages
*
* @help
*
* Set random Battle Messages to the following in the parameter
* to take 1 random each time you enter a battle.
*
* Each time you get the upper hand, a text is shown at random.
* Each time you get a surprise attack, a text is shown at random.
* Each time you lost a battle, a text is shown at random.
* Each time you win a battle, a text is shown at random.
* Each time you try to escape (success), a text is shown at random.
* Each time you try to escape (failure), a text is shown at random.
*
* No more the same message over and over again, now it's YOUR choice,
* what text is shown.
*
* You can also disable the text entirely for those, so you don't
* see the text for Emerge, start, lost, win or escape.
*
* === TERMS OF USE ===
* Free for Non-Commercial and Commercial use when credit is given.
* credit one of the following:
* ShadowDragon 
* ShadowDragonJB
*
* Version + UPDATE'S
* 3-03-2023 Version 0.1.0 first release
* 12-03-2003 Version 0.2.0 added option to hide battle messages or not.
*
* === WARNING ===
* YOU ARE NOT ALLOWED TO MODIFY, EXTEND, REDISTRIBUTE OR SELL IT.
*
* @param Enable Messages?
*
* @param Start Messages
* @parent Enable Messages?
* @desc Show Start Messages?
* @type boolean
* @on Show
* @off Hide
* @default true
* 
* @param Defeat Messages
* @parent Enable Messages?
* @desc Show Defeat Messages?
* @type boolean
* @on Show
* @off Hide
* @default true
*
* @param Victory Messages
* @parent Enable Messages?
* @desc Show Victory Messages?
* @type boolean
* @on Show
* @off Hide
* @default true
* 
* @param Escape Success Message
* @parent Enable Messages?
* @desc Show Escape Success Messages?
* @type boolean
* @on Show
* @off Hide
* @default true
*
* @param Escape Failure Message
* @parent Enable Messages?
* @desc Show Escape Failure Messages?
* @type boolean
* @on Show
* @off Hide
* @default true
*
* @param Enemy Emerges
* @desc Show Enemy names or Troop Name?
* @type boolean
* @on Troop Names
* @off Enemies Names
* @default false
*
* @param StartMessages_p
* @desc set Battle Start Preemptive Messages
* @type []
* @default
*
* @param StartMessages_s
* @desc set Battle Start Surprise Messages
* @type []
* @default
*
* @param DefeatMessages
* @desc set Battle Defeat Messages
* @type []
*
* @param VictoryMessages
* @desc set Battle Victory Messages
* @type []
*
* @param EscapeSuccessMessages
* @desc set Battle Escape Success Messages
* @type []
*
* @param EscapeFailureMessages
* @desc set Battle Escape Failure Messages
* @type []
*
* @param Window Battle Log
*
* @param Skip Battle Log
* @parent Window Battle Log
* @desc Skip Battle Log Window?
* @type boolean
* @on Yes
* @off No
* @default true
*
* @param Skip Delay
* @parent Window Battle Log
* @desc Skip Battle Log Window?
* @type boolean
* @on Yes
* @off No
* @default true
*
*/


var ShadowDragon = ShadowDragon || {};          // Local NameSpace
    ShadowDragon.Battlemessage = ShadowDragon.Battlemessage || {};    // Local Plugin NameSpace

var SDJBbm = ShadowDragon.Battlemessage;                     // Plugin NameSpace shortcut
    SDJBbm.version = 0.3;                       // Version Number
    SDJBbm.pluginName = "SDJB_Battlemessages";        // Plugin Name

var Imported = Imported || {};                  // import Plugin
    Imported[SDJBbm.pluginName] = true;           // Import Plugin Name


//(() => {
    'use strict'

    SDJBparam = PluginManager.parameters("SDJB_Battlemessages");

    // Battle Manager
    SDJBbm._showStart = (SDJBparam["Start Messages"] === "true");
    SDJBbm._showDefeat = (SDJBparam["Defeat Messages"] === "true");
    SDJBbm._showVictory = (SDJBparam["Victory Messages"] === "true");
    SDJBbm._showEscapeS = (SDJBparam["Escape Success Message"] === "true");
    SDJBbm._showEscapeF = (SDJBparam["Escape Failure Message"] === "true");
    SDJBbm._start_b = (SDJBparam["Enemy Emerges"] === "true");
    SDJBbm._start_p = JSON.parse(SDJBparam.StartMessages_p);
    SDJBbm._start_s = JSON.parse(SDJBparam.StartMessages_s);
    SDJBbm._defeat = JSON.parse(SDJBparam.DefeatMessages);
    SDJBbm._victory = JSON.parse(SDJBparam.VictoryMessages);
    SDJBbm._escSuccess = JSON.parse(SDJBparam.EscapeSuccessMessages);
    SDJBbm._escFailure = JSON.parse(SDJBparam.EscapeFailureMessages);

    // Window Battle Log
    SDJBbm._controller = false; //Controller must be false to make delay work correctly
    SDJBbm._skipBlog = (SDJBparam["Skip Battle Log"] === "true");
    SDJBbm._skipDelay = (SDJBparam["Skip Delay"] === "true");

    // === Battle Manager === \\
    BattleManager.displayStartMessages = function() {
        if (SDJBbm._showStart) {
            if (SDJBbm._start_b) {
                $gameMessage.add(TextManager.emerge.format($gameTroop.troop().name));   
            } else {
                $gameTroop.enemyNames().forEach(function(name) {
                    $gameMessage.add(TextManager.emerge.format(name));
                });
            }
            if (this._preemptive) {
                let preemptive = SDJBbm._start_p;
                let dSpM = preemptive[Math.randomInt(preemptive.length)];
                $gameMessage.add(dSpM);
            } else if (this._surprise) {
                let surprise = SDJBbm._start_s;
                let dSsM = surprise[Math.randomInt(surprise.length)];
                $gameMessage.add(dSsM);
            }
        }
    }; 

    BattleManager.displayDefeatMessage = function() {
        if (SDJBbm._showDefeat) {
            let defeat = SDJBbm._defeat;
            let dDM = defeat[Math.randomInt(defeat.length)];
            $gameMessage.add(dDM);
        }
    };

    BattleManager.displayVictoryMessage = function() {
        if (SDJBbm._showVictory) {
            let victory = SDJBbm._victory;
            let dVM = victory[Math.randomInt(victory.length)];
            $gameMessage.add(dVM);
        }
    };

    BattleManager.displayEscapeSuccessMessage = function() {
        if (SDJBbm._showEscapeS) {
            let escSuccess = SDJBbm._escSuccess;
            let dESM = escSuccess[Math.randomInt(escSuccess.length)];
            $gameMessage.add(dESM);
        }
    };

    BattleManager.displayEscapeFailureMessage = function() {
        if (SDJBbm._showEscapeF) {
            let escFailure = SDJBbm._escFailure;
            let dEFM = escFailure[Math.randomInt(escFailure.length)];
            $gameMessage.add(dEFM);
        }
    }; 

    // Window BattleLog
    SDJBbm._WB_isBusy = Window_BattleLog.prototype.isBusy;
        Window_BattleLog.prototype.isBusy = function() {
	        if (!SDJBbm._controller || !SDJBbm._skipBlog || !SDJBbm._skipDelay){
	            return SDJBbm._WB_isBusy.call(this);
	        }
	        return false;
        };
 
    SDJBbm._WB_setWaitMode = Window_BattleLog.prototype.setWaitMode;
        Window_BattleLog.prototype.setWaitMode = function(waitMode) {
            if (!SDJBbm._skipDelay){
                SDJBbm._WB_setWaitMode.call(this, waitMode);
            }
        };

    SDJBbm._WB_wait = Window_BattleLog.prototype.wait;
        Window_BattleLog.prototype.wait = function() {
            if (!SDJBbm._skipDelay){
                SDJBbm._WB_wait.call(this);
            }
        };


    SDJBbm._WB_addText = Window_BattleLog.prototype.addText;
        Window_BattleLog.prototype.addText = function(text) {
	        if (!SDJBbm._skipBlog){
                SDJBbm._WB_addText.call(this, text);
            }
        };

//})();
