// XPStyleResult.js Ver.1.1.0

/*:
* @target MZ
* @plugindesc RMXP风格战斗结算
* @author あわやまたな (Awaya_Matana)
* @url https://awaya3ji.seesaa.net/article/499948976.html
* @help Ver.1.1.0
* 您可以使用向上/向下键、鼠标滚轮、拖动和滑动来滚动窗口。
*
* @param terms
* @text 条件
* @desc 设置条条件
* @type struct<terms>
* @default {"levelUp":"升级!"}
*
* @param x
* @text X-坐标
* @desc X coordinate of result window
* @default auto
* @type combo
* @option auto
*
* @param y
* @text Y-坐标
* @desc Y coordinate of result window
* @default auto
* @type combo
* @option auto
*
* @param width
* @text 宽度
* @desc Width of result window
* @default auto
* @type combo
* @option auto
*
* @param maxLines
* @text 最大行数
* @desc 最大行数
* @default 7
* @type number
* @min 1
*
* @param countItems
* @text 清点物品整理
* @desc 清点物品
* @default true
* @type boolean
*
* @param noRepeat
* @text 翻页时无重复
* @desc Never miss a text with a long press.
* @default true
* @type boolean
*
* @param windowSkin
* @text 窗口皮肤
* @desc 指定结果窗口的文件名。
* @type file
* @dir img/system
*
* @param windowOpacity
* @text 窗口不透明度
* @desc Specifies the opacity of the result window.
* -1 is system default.
* @default -1
* @type number
* @min -1
*
*/

/*:ja
* @target MZ
* @plugindesc リザルトをツクールXP風にします。
* @author あわやまたな (Awaya_Matana)
* @url https://awaya3ji.seesaa.net/article/499948976.html
* @help キー上下、マウスホイール、ドラッグ、スワイプでウィンドウを
* スクロールできます。
*
* [更新履歴]
* 2023/07/06：Ver.1.0.0　公開。
* 2023/07/25：Ver.1.1.0　挙動修正。
*
* @param terms
* @text 用語
* @desc 用語を設定します。
* @type struct<terms>
* @default {"levelUp":"LEVEL UP!"}
*
* @param x
* @text X座標
* @desc リザルトウィンドウのX座標
* @default auto
* @type combo
* @option auto
*
* @param y
* @text Y座標
* @desc リザルトウィンドウのY座標
* @default auto
* @type combo
* @option auto
*
* @param width
* @text 幅
* @desc リザルトウィンドウの幅
* @default auto
* @type combo
* @option auto
*
* @param maxLines
* @text 最大行数
* @desc リザルトウィンドウの最大行数
* @default 7
* @type number
* @min 1
*
* @param countItems
* @text アイテムを数える
* @desc アイテムを数えます。
* @default true
* @type boolean
*
* @param noRepeat
* @text ページ送り時リピートなし
* @desc 長押しでテキストを見逃さないようにします。
* @default true
* @type boolean
*
* @param windowSkin
* @text ウィンドウスキン
* @desc リザルトウィンドウのファイル名を指定します。
* @type file
* @dir img/system
*
* @param windowOpacity
* @text ウィンドウ不透明度
* @desc リザルトウィンドウの不透明度を指定します。
* -1でシステムの初期値。
* @default -1
* @type number
* @min -1
*
*/

/*~struct~terms:
*
* @param levelUp
* @text Level Up
* @default LEVEL UP!
*
*/

/*~struct~terms:ja
*
* @param levelUp
* @text レベルアップ
* @default LEVEL UP!
*
*/

'use strict';

{
	//プラグイン名取得。
	const script = document.currentScript;
	const pluginName = document.currentScript.src.match(/^.*\/(.*).js$/)[1];
	const hasPluginCommonBase = typeof PluginManagerEx === "function";
	const fesStyleBattle = $plugins.some(plugin => Utils.extractFileName(plugin.name) === "FesStyleBattle" && plugin.status);

	const parameters = PluginManager.parameters(pluginName);

	const terms = JSON.parse(parameters["terms"]);
	const resultWindowX = parameters["x"] === "auto" ? parameters["x"] : Number(parameters["x"]);
	const resultWindowY = parameters["y"] === "auto" ? parameters["y"] : Number(parameters["y"]);
	const resultWindowWidth = parameters["width"] === "auto" ? parameters["width"] : Number(parameters["width"]);
	const maxLines = Number(parameters["maxLines"]);
	const messageSpeed = Number(parameters["messageSpeed"]);
	const countItems = parameters["countItems"] === "true";
	const noRepeat = parameters["noRepeat"] === "true";
	const pictureId = Number(parameters["pictureId"]);
	const windowSkin = parameters["windowSkin"];
	const windowOpacity = Number(parameters["windowOpacity"]);

	//-----------------------------------------------------------------------------
	// Scene_Battle

	const _Scene_Battle_createAllWindows = Scene_Battle.prototype.createAllWindows;
	Scene_Battle.prototype.createAllWindows = function() {
		_Scene_Battle_createAllWindows.call(this);
		this.createResultWindow();
	};

	Scene_Battle.prototype.createResultWindow = function() {
		const rect = this.resultWindowRect();
		this._resultWindow = new Window_Result(rect);
		this.addWindow(this._resultWindow);
		this._resultWindow.setSpriteset(this._spriteset);
	};

	Scene_Battle.prototype.resultWindowRect = function() {
		let ww = resultWindowWidth;
		if (ww === "auto") {
			ww = Graphics.boxWidth / 2;
		}
		const wh = this.calcWindowHeight(maxLines, false);
		let ox = resultWindowX;
		if (ox === "auto") {
			ox = 0;
		}
		let oy = resultWindowY;
		if (oy === "auto") {
			oy = -this.statusWindowRect().height / 2;
		}
		const wx = (Graphics.boxWidth - ww) / 2 + ox;
		const wy = (Graphics.boxHeight - wh) / 2 + oy;
		return new Rectangle(wx, wy, ww, wh);
	};

	const _Scene_Battle_stop = Scene_Battle.prototype.stop;
	Scene_Battle.prototype.stop = function() {
		_Scene_Battle_stop.call(this);
		this._statusWindow.open();
	};

	const _Scene_Battle_updateStatusWindowVisibility = Scene_Battle.prototype.updateStatusWindowVisibility;
	Scene_Battle.prototype.updateStatusWindowVisibility = function() {
		const resultMode = $gameMessage._resultMode;
		$gameMessage._resultMode = false;
		_Scene_Battle_updateStatusWindowVisibility.call(this);
		$gameMessage._resultMode = resultMode;
	};

	const _Scene_Battle_shouldOpenStatusWindow = Scene_Battle.prototype.shouldOpenStatusWindow;
	Scene_Battle.prototype.shouldOpenStatusWindow = function() {
		return _Scene_Battle_shouldOpenStatusWindow.call(this) || BattleManager._phase === "battleEnd";
	};

	//-----------------------------------------------------------------------------
	// Game_Message

	const _Game_Message_clear = Game_Message.prototype.clear;
	Game_Message.prototype.clear = function() {
		_Game_Message_clear.call(this);
		this._resultMode = false;
	};

	Game_Message.prototype.resultMode = function() {
		return this._resultMode;
	};

	Game_Message.prototype.setResult = function() {
		this._resultMode = true;
	};

	const _Game_Message_isBusy = Game_Message.prototype.isBusy;
	Game_Message.prototype.isBusy = function() {
		return  _Game_Message_isBusy.call(this) || this.resultMode();
	};

	//-----------------------------------------------------------------------------
	// Window_Result

	function Window_Result() {
		this.initialize(...arguments);
	}

	Window_Result.prototype = Object.create(Window_Scrollable.prototype);
	Window_Result.prototype.constructor = Window_Result;

	Window_Result.prototype.initialize = function(rect) {
		Window_Scrollable.prototype.initialize.call(this, rect);
		this.cursorVisible = false;
		this._lines = [];
		this._methods = [];
		this._waitCount = 0;
		this._waitMode = "";
		this._baseLineStack = [];
		this._spriteset = null;
		this.refresh();
		this.openness = 255;
		this._inputForNext = false;
		this.hide();
	};

	Window_Result.prototype.setSpriteset = function(spriteset) {
		this._spriteset = spriteset;
	};

	Window_Result.prototype.updateArrows = function() {
		Window_Scrollable.prototype.updateArrows.call(this);
		if (!this.isScrollEnabled()) {
			this.downArrowVisible = this.upArrowVisible = false;
		}
	};

	Window_Result.prototype.isScrollEnabled = function() {
		return this._inputForNext;
	};

	if (windowSkin) {
		Window_Result.prototype.loadWindowskin = function() {
			this.windowskin = ImageManager.loadSystem(windowSkin);
		};
	}

	if (windowOpacity > -1) {
		Window_Result.prototype.updateBackOpacity = function() {
			this.backOpacity = windowOpacity;
		};
	}

	Window_Result.prototype.setSpriteset = function(spriteset) {
		this._spriteset = spriteset;
	};

	Window_Result.prototype.maxLines = function() {
		return this.numLines();
	};

	Window_Result.prototype.numLines = function() {
		return this._lines.length;
	};

	Window_Result.prototype.messageSpeed = function() {
		return messageSpeed;
	};

	Window_Result.prototype.isBusy = function() {
		return this._waitCount > 0 || this._waitMode || this._methods.length > 0;
	};

	Window_Result.prototype.update = function() {
		Window_Scrollable.prototype.update.call(this);
		while (!this.updateWait() && this._methods.length > 0) {
			this.callNextMethod();
		}
		this.processButtonScroll();
	};

	Window_Result.prototype.processButtonScroll = function() {
		if (this.isScrollEnabled() && this._scrollDuration === 0) {
			if (Input.isPressed("down")) {
				this.smoothScrollDown(1);
			}
			if (Input.isPressed("up")) {
				this.smoothScrollUp(1);
			}
		}
	};

	Window_Result.prototype.updateWait = function() {
		return this.updateWaitCount() || this.updateWaitMode();
	};

	Window_Result.prototype.updateWaitCount = function() {
		if (this._waitCount > 0) {
			this._waitCount -= this.isFastForward() ? 3 : 1;
			if (this._waitCount < 0) {
				this._waitCount = 0;
			}
			return true;
		}
		return false;
	};

	Window_Result.prototype.isTriggered = function() {
		let triggered = false;
		if (noRepeat && this._inputForNext) {
			triggered = Input.isTriggered("ok") || Input.isTriggered("cancel");
		} else {
			triggered = Input.isRepeated("ok") || Input.isRepeated("cancel");
		}
		if (triggered) {
			this._lastRepeated = false;
		}
		return triggered || this.isTouchTriggered();
	};

	Window_Result.prototype.isTouchTriggered = function() {
		const isScrollEnabled = this.isScrollEnabled();
		const repeated = !isScrollEnabled && TouchInput.isRepeated();
		let triggered = isScrollEnabled && TouchInput.isClicked();

		if (isScrollEnabled && this._lastRepeated && !TouchInput.isPressed()) {
			triggered = false;
			this._lastRepeated = false;
		}
		if (!isScrollEnabled) {
			this._lastRepeated = repeated;
		}
		return triggered || repeated;
	};

	Window_Result.prototype.updateWaitMode = function() {
		let waiting = false;
		switch (this._waitMode) {
			case "input":
				waiting = !this.isTriggered();
				if (!waiting) {
					SoundManager.playOk();
					this._inputForNext = false;
				}
				if (!this._inputForNext) {
					this.pause = waiting;
				}
				break;
			case "effect":
				waiting = this._spriteset.isEffecting();
				break;
			case "scroll":
				waiting = !!this._scrollDuration;
				break;
			case "movement":
				waiting = this._spriteset.isAnyoneMoving();
				break;
			case "statusMovement":
				waiting = SceneManager._scene.statusWindowX() !== SceneManager._scene._statusWindow.x;
				break;
		}
		if (!waiting) {
			this._waitMode = "";
		}
		return waiting;
	};

	Window_Result.prototype.setWaitMode = function(waitMode) {
		this._waitMode = waitMode;
	};

	Window_Result.prototype.callNextMethod = function() {
		if (this._methods.length > 0) {
			const method = this._methods.shift();
			if (method.name && this[method.name]) {
				this[method.name].apply(this, method.params);
			} else {
				throw new Error("Method not found: " + method.name);
			}
		}
	};

	Window_Result.prototype.nextMethodName = function() {
		const method = this._methods[0];
		if (method) {
			return method.name || "";
		}
		return "";
	};

	Window_Result.prototype.isFastForward = function() {
		return (
			Input.isLongPressed("ok") ||
			Input.isPressed("shift") ||
			TouchInput.isLongPressed()
		);
	};

	Window_Result.prototype.push = function(methodName) {
		const methodArgs = Array.prototype.slice.call(arguments, 1);
		this._methods.push({ name: methodName, params: methodArgs });
	};

	Window_Result.prototype.clear = function() {
		this._lines = [];
		this._baseLineStack = [];
		this.refresh();
		this.scrollTo(0,0);
	};

	Window_Result.prototype.wait = function() {
		this._waitCount = this.messageSpeed();
	};

	Window_Result.prototype.setWait = function(duration) {
		this._waitCount = duration;
	};

	Window_Result.prototype.waitForEffect = function() {
		this.setWaitMode("effect");
	};

	Window_Result.prototype.waitForMovement = function() {
		this.setWaitMode("movement");
	};

	Window_Result.prototype.waitForStatusMovement = function() {
		this.setWaitMode("statusMovement");
	};

	Window_Result.prototype.waitForInput = function(next = false) {
		this.setWaitMode("input");
		this._inputForNext = next;
	};

	Window_Result.prototype.waitForScroll = function() {
		this.setWaitMode("scroll");
	};

	Window_Result.prototype.waitForLoad = function() {
		this.setWaitMode("loadBitmap");
	};

	Window_Result.prototype.addText = function(text, align = "left") {
		const data = { "type": "text", "text": text, "align": align };
		this._lines.push(data);
		this.refresh();
	};

	Window_Result.prototype.addTextEx = function(text) {
		const data = { "type": "textEx", "text": text };
		this._lines.push(data);
		this.refresh();
	};

	Window_Result.prototype.addDropItem = function(item, num) {
		const data = { "type": "dropItem", "item": item, "num": num };
		this._lines.push(data);
		this.refresh();
	};

	Window_Result.prototype.addGainExp = function(exp, gold) {
		const data = { "type": "gainExp", "exp": exp, "gold": gold };
		this._lines.push(data);
		this.refresh();
	};

	Window_Result.prototype.pushBaseLine = function() {
		this._baseLineStack.push(this._lines.length);
	};

	Window_Result.prototype.popBaseLine = function() {
		const baseLine = this._baseLineStack.pop();
		while (this._lines.length > baseLine) {
			this._lines.pop();
		}
	};

	Window_Result.prototype.waitForNewLine = function() {
		let baseLine = 0;
		if (this._baseLineStack.length > 0) {
			baseLine = this._baseLineStack[this._baseLineStack.length - 1];
		}
		if (this._lines.length > baseLine) {
			this.wait();
		}
	};

	Window_Result.prototype.overallHeight = function() {
		return this.numLines() * this.itemHeight();
	};

	Window_Result.prototype.start = function(numLines) {
		this.updatePlacement(numLines);
		this.createContents();
		this.refresh();
		this._lastRepeated = noRepeat;
	};

	Window_Result.prototype.updatePlacement = function(numLines) {
		this.height = this.windowHeight(numLines);
		let oy = resultWindowY;
		if (oy === "auto") {
			oy = -(SceneManager._scene.statusWindowRect().height / 2);
		}
		this.y = (Graphics.boxHeight - this.height) / 2 + oy;
	};

	Window_Result.prototype.windowHeight = function(numLines) {
		return this.fittingHeight(numLines);
	};

	Window_Result.prototype.end = function() {
		if (!this.isBusy()) {
			if (BattleManager._phase === "battleEnd") {
				this._inputForNext = true;
				$gameMessage.clear();
			} else {
				this.push("terminateResult");
			}
		}
	};

	Window_Result.prototype.terminateResult = function() {
		this.hide();
		$gameMessage.clear();
		this.clear();
		$gameScreen.erasePicture(pictureId);
	};

	Window_Result.prototype.lineRect = function(index) {
		const itemHeight = this.itemHeight();
		const padding = this.itemPadding();
		const x = padding;
		const y = index * itemHeight - this.scrollBaseY();
		const width = this.innerWidth - padding * 2;
		const height = itemHeight;
		return new Rectangle(x, y, width, height);
	};

	Window_Result.prototype.topRow = function() {
		return Math.floor(this.scrollY() / this.itemHeight());
	};

	Window_Result.prototype.topIndex = function() {
		return this.topRow();
	};

	Window_Result.prototype.refresh = function() {
		this.contents.clear();
		this.drawAllLines();
	};

	Window_Result.prototype.drawAllLines = function() {
		const topIndex = this.topIndex();
		for (let i = 0; i < this.maxVisibleItems(); i++) {
			const index = topIndex + i;
			if (index < this.maxLines()) {
				this.drawLineText(index);
			}
		}
	};

	Window_Result.prototype.drawLineText = function(index) {
		const rect = this.lineRect(index);
		this.contents.clearRect(rect.x, rect.y, rect.width, rect.height);
		const data = this._lines[index];
		if (data.type === "textEx") {
			this.drawTextEx(data.text, rect.x, rect.y, rect.width);
		} else if (data.type === "text") {
			this.drawText(data.text, rect.x, rect.y, rect.width, data.align);
		} else if (data.type === "dropItem") {
			this.drawDropItem(index);
		} else if (data.type === "gainExp") {
			this.drawGainExp(index);
		}
	};

	Window_Result.prototype.drawGainExp = function(index) {
		const rect = this.lineRect(index);
		const data = this._lines[index];
		this.resetTextColor();
		let textWidth = this.textWidth(data.exp);
		this.drawText(data.exp, rect.x, rect.y);
		rect.x += textWidth + this.textWidth(" ")/2;
		textWidth = this.textWidth(TextManager.expA);
		this.changeTextColor(ColorManager.systemColor());
		this.drawText(TextManager.expA, rect.x, rect.y);
		rect.x += textWidth + this.textWidth("  ");
		textWidth = this.textWidth(data.gold);
		this.resetTextColor();
		this.drawText(data.gold, rect.x, rect.y);
		rect.x += textWidth + this.textWidth(" ")/2;
		this.changeTextColor(ColorManager.systemColor());
		this.drawText(TextManager.currencyUnit, rect.x, rect.y);
	};

	Window_Result.prototype.drawDropItem = function(index) {
		const rect = this.lineRect(index);
		const data = this._lines[index];
		const item = data.item;
		const num = data.num;
		const numberWidth = num ? this.numberWidth(rect.width) : 0;
		const nameWidth = rect.width - numberWidth;
		this.drawItemName(item, rect.x, rect.y, nameWidth);
		if (num) {
			this.drawItemNumber(num, rect.x + nameWidth, rect.y, numberWidth);
		}
	};

	Window_Result.prototype.drawItemNumber = function(num, x, y, width) {
		this.drawText(`✕${num}`.padEnd(3, " "), x, y, width);
	};

	Window_Result.prototype.numberWidth = function(width) {
		return this.textWidth(`✕`.padEnd(3, " "));
	};

	Window_Result.prototype.paint = function() {
		this.refresh();
	};

	Window_Result.prototype.maxVisibleItems = function() {
		const visibleRows = Math.ceil(this.contentsHeight() / this.itemHeight());
		return visibleRows;
	};

	Window_Result.prototype.contentsHeight = function() {
		return this.innerHeight + this.itemHeight();
	};

	Window_Result.prototype.createResultMethods = function(data) {
		this.push("show");
		this.push("start", Math.min(data.length, maxLines));
		this.push("clear");
		data.forEach((data, index) => {
			const line = index + 1;
			if (typeof data === 'object') {
				if ("item" in data) {
					this.push("addDropItem", data.item, data.num);
				} else if ("exp" in data) {
					this.push("addGainExp", data.exp, data.gold);
				} else {
					this.push("addText", data.text, "right");
				}
			} else {
				this.push("addTextEx", data);
			}
		});
		this.push("waitForInput", true);
		this.push("end");
	};

	//-----------------------------------------------------------------------------
	// Window_BattleStatus

	Window_BattleStatus.prototype.drawLevelUp = function(index) {
		const actor = this.actor(index);
		const rect = this.itemRectWithPadding(index);
		const nameX = this.nameX(rect);
		const nameY = this.nameY(rect);
		this.placeLevelUp(actor, nameX, rect.y);
	};

	Window_BattleStatus.prototype.placeLevelUp = function(actor, x, y) {
		const key = "actor%1-levelUp".format(actor.actorId());
		const sprite = this.createInnerSprite(key, Sprite_LevelUpXp);
		sprite.setup(actor);
		sprite.move(x, y);
		sprite.show();
	};

	//-----------------------------------------------------------------------------
	// Game_Actor

	Game_Actor.prototype.shouldDisplayLevelUp = function() {
		return false;
	};

	//-----------------------------------------------------------------------------
	// BattleManager

	const _BattleManager_isBusy = BattleManager.isBusy;
	BattleManager.isBusy = function() {
		return _BattleManager_isBusy.call(this) || SceneManager._scene._resultWindow.isBusy();
	};

	const _BattleManager_gainExp = BattleManager.gainExp;
	BattleManager.gainExp = function() {
		this._actorLevels = [];
		$gameParty.battleMembers().forEach(actor => this._actorLevels.push(actor.level));
		_BattleManager_gainExp.call(this);
		const statusWindow = SceneManager._scene._statusWindow;
		$gameParty.battleMembers().forEach((actor, index) => {
			if (actor.level !== this._actorLevels[index]) {
				statusWindow.drawLevelUp(index)
			}
		});
	};

	BattleManager.displayExp = function() {
		$gameMessage.setResult();
		const exp = this._rewards.exp;
		const gold = this._rewards.gold;
		const texts = [];
		const fontSize = $gameSystem.mainFontSize();
		texts.push({"exp": exp, "gold": gold});

		const items = this._rewards.items;

		if (countItems) {
			const numData = [];
			for (const item of items) {
				const index = numData.findIndex(data => data.item === item)
				if (index === -1) {
					numData.push({"item": item, "num": 1});
				} else {
					numData[index].num++;
				}
			};

			if (items.length > 0) {
				const resultWindow = SceneManager._scene._resultWindow;
				const data = [];
				for (const numItems of numData) {
					const item = numItems.item;
					const num = numItems.num;
					texts.push( { "item": item, "num": num } );
				}
			}
		} else {
			for (const item of items) {
				texts.push( { "item": item } );
			};
		}

		const resultWindow = SceneManager._scene._resultWindow;
		if (fesStyleBattle) {
			resultWindow.push("waitForStatusMovement");
		}
		resultWindow.createResultMethods(texts, "exp");
	};

	BattleManager.displayVictoryMessage = function() {};
	BattleManager.displayGold = function() {};
	BattleManager.displayDropItems = function() {};

	//-----------------------------------------------------------------------------
	// Sprite_LevelUpXp

	function Sprite_LevelUpXp() {
		this.initialize(...arguments);
	}

	Sprite_LevelUpXp.prototype = Object.create(Sprite_Name.prototype);
	Sprite_LevelUpXp.prototype.constructor = Sprite_LevelUpXp;

	Sprite_LevelUpXp.prototype.textColor = function() {
		return ColorManager.normalColor();
	};

	Sprite_LevelUpXp.prototype.name = function() {
		return terms.levelUp;
	};

	window.Sprite_LevelUpXp = Sprite_LevelUpXp;

}